@@ -10,12 +10,12 @@ from ..common.misc import OrderedDict
 from ..common.grammar_types import mixed
 from .options import Option
 import pyparsing as pp
-from .base_configuration import BaseConfiguration
+from .configuration import Configuration
 import itertools
 import re
 from typing import Union, Any, Dict
 
-class ConfigurationContainer(BaseConfiguration):
+class ConfigurationContainer(Configuration):
   """ A container for configuration (problem-definition) options and/or sections.
 
   Options in the configuration (problem-definition) files are grouped to
@@ -27,18 +27,21 @@ class ConfigurationContainer(BaseConfiguration):
   def __init__(self, definition, container=None):
       """ Create the container and its members, according to the definition """
       super().__init__(definition, container)
-      self._members = OrderedDict()
       """
       The members of the container, in a form of ``{obj.name : obj}``
       """
-      self._interactive_members = OrderedDict()
+      self._init_members_from_the_definition()
+
+  def _init_members_from_the_definition(self):
+      self._members = OrderedDict()
       """
       Non-hidden members of the containers, accesible via sanitized names.
       I.e. via names with whitespaces and other special characters replaced by underscore.
       These sanitized names are then used as names for "attributes" of this container, to
       make the members accesible via ``<container>.<member>`` notation.
       """
-      for v in definition.members():
+      self._interactive_members = OrderedDict()
+      for v in self._definition.members():
           self._add(v.create_object(self))
 
   def members(self):
@@ -272,7 +275,7 @@ class ConfigurationContainer(BaseConfiguration):
               out[i.name] = value
       return out or None
 
-  def to_string(self, *, validate=False):
+  def to_string(self):
       """
       Return the configuration (problem definition) in a string.
 
@@ -283,7 +286,7 @@ class ConfigurationContainer(BaseConfiguration):
       """
       from io import StringIO
       s = StringIO()
-      self.save_to_file(s, validate=validate)
+      self._save_to_file(s)
       return s.getvalue()
 
   def _find_value(self, name):
@@ -315,7 +318,7 @@ class ConfigurationContainer(BaseConfiguration):
       The sanitized names are keys in ``interactive_members`` array, and thus
       the members are accesible by ``<container>.<member>`` notation.
       """
-      return re.sub(r'[-\s.]','_',name)
+      return re.sub(r'[()]','', re.sub(r'[-\s.]','_',name))
 
   def _add(self, member):
       name = member.name
@@ -325,14 +328,18 @@ class ConfigurationContainer(BaseConfiguration):
           if not iname in self._interactive_members:
               self._interactive_members[iname] = member
 
-  def save_to_file(self, file, *, validate=True):
+  def is_changed(self):
+      for i in self:
+          if i.is_changed(): return True
+      return False
+
+  def _save_to_file(self, file)->bool:
       """ Save the configuration to a file. The method is implemented in the descendants.
 
       TODO
       ----
       The implementations in the descendant could be probably merged.
       """
-
       raise NotImplemented()
 
   def __setattr__(self, name, value):
@@ -345,6 +352,34 @@ class ConfigurationContainer(BaseConfiguration):
         val = self._get_member(name)
         val.set(value)
 
+  def validate(self, why:str='save'):
+      """ Validate the configuration data. Raise an exception, if the validation fail.
+
+      Parameters
+      ----------
+      why
+        Type of the validation. Possible values
+        ``save`` - Full validation, during save.
+        ``set`` - Validation on user input. Allow required values not to be set.
+        ``parse`` - Validation during parsing - some check, that are enforced by the parser, can be skipped.
+      """
+      self._definition.validate(DictAdaptor(self), why)
+      for o in self:
+          o.validate(why)
+
+
+class DictAdaptor:
+  """ This class wraps a container to behave as a read-only dict """
+
+  def __init__(self, container):
+      self.container=container
+
+  def __hasitem__(self, name):
+      return self.container__hasitem__(name)
+
+  def __getitem__(self, name):
+      return self.container.__getitem__(name)()
+
 class BaseSection(ConfigurationContainer):
   """ A section of SPRKKR configuration - i.e. part of the configuration file. """
 
@@ -362,25 +397,48 @@ class BaseSection(ConfigurationContainer):
            return True
       return False
 
-  def save_to_file(self, file, *, validate=True):
+  def validate(self, why:str='save'):
+      """ Validate the configuration data. Raise an exception, if the validation fail.
+
+      Parameters
+      ----------
+      why
+        Type of the validation. Possible values
+        ``save`` - Full validation, during save.
+        ``set`` - Validation on user input. Allow required values not to be set.
+        ``parse`` - Validation during parsing - some check, that are enforced by the parser, can be skipped.
+      """
+      if why == 'save' and not self._definition.is_optional and not self.has_any_value():
+          raise ValueError(f"Non-optional section {self._definition.name} has no value to save")
+      super().validate(why)
+
+  def _save_to_file(self, file)->bool:
       """ Save the content of the container to the file (according to the definition)
 
       Parameters
       ----------
       file: file
         File object (open for writing), where the data should be written
+
+      Returns
+      -------
+      something_have_been_written
+        If any value have been written return True, otherwise return False.
       """
-      if not self.has_any_value():
-         if validate and not self._definition.is_optional:
-            raise ValueError(f"Non-optional section {self._definition.name} has no value to save")
-         return
+      d = self._definition
+      if d.is_expert:
+          if not self.is_changed():
+              return False
+      else:
+          if not self.has_any_value():
+              return False
       if self._definition.name_in_grammar:
          file.write(self._definition.name)
          file.write('\n')
       for o in self:
-          if o.save_to_file(file, validate=validate):
+          if o._save_to_file(file):
              file.write(self._definition.delimiter)
-      file.flush()
+      return True
 
 class Section(BaseSection):
   """ A standard section of a task or potential (whose content is predefinded by SectionDefinition) """
@@ -391,7 +449,7 @@ class Section(BaseSection):
 
       Returns
       -------
-      ase2sprkkr.common.configuration_definitions.BaseContainerDefinition
+      ase2sprkkr.common.configuration_definitions.ContainerDefinition
       The definition of the section. I.e. the object that defines, which configuration values
       are in the section, their default values etc.
       """
@@ -435,32 +493,17 @@ class RootConfigurationContainer(ConfigurationContainer):
   In addition to container capabilities, it can read/write its data from/to file.
   """
 
-  def save_to_file(self, file, *, validate=True):
-      """ Save the configuration to a file in a format readable by SPR-KKR i
-
-      Parameters
-      ----------
-      file: str or file
-        File to read the data from
-
-      validate: bool
-        Validate the data in the container first and raise an exception,
-        if there is an error (e.g. the the data are not complete)
-      """
-      if not hasattr(file, 'write'):
-         with open(file, "w") as file:
-           return self.save_to_file(file, validate=validate)
-
+  def _save_to_file(self, file):
+      """ Implementation of the saving the configuration """
       it = iter(self)
       i = next(it)
       if i:
-        i.save_to_file(file, validate=validate)
+        i._save_to_file(file)
         for i in it:
           file.write(self._definition.delimiter)
-          i.save_to_file(file, validate=validate)
-      file.flush()
+          i._save_to_file(file)
 
-  def read_from_file(self, file, clear_first=True):
+  def read_from_file(self, file, clear_first:bool=True, allow_dangerous:bool=False):
       """ Read data from a file
 
       Parameters
@@ -468,16 +511,14 @@ class RootConfigurationContainer(ConfigurationContainer):
       file: str or file
         File to read the data from
 
-      clear_first: bool
+      clear_first
         Clear the container first.
         Otherwise, the data in the sections that are not present in the
         file are preserved.
+      allow_dangerous
+        Allow to load dangerous_values, i.e. the values that do not pass the requirements for the input values (e.g. of a different type or constraint-violating)
       """
-      values = self._definition.parse_file(file)
-      #except Exception as e:
-      #   print(e)
-      #   breakpoint()
-      #   print(e)
+      values = self._definition.parse_file(file, allow_dangerous=allow_dangerous)
       if clear_first:
          self.clear(True)
       self.set(values, unknown='add')
