@@ -4,6 +4,7 @@ These configuration parameters are supplied to SPR-KKR executables as input file
 The containers are also the objects, that take care about executing the SPR-KKR
 executables.
 """
+from __future__ import annotations
 
 import os
 import io
@@ -13,11 +14,15 @@ import importlib
 from . import definitions
 from ..outputs import readers
 from ..outputs.readers.default import DefaultOutputReader
-from ..common.configuration_containers import RootConfigurationContainer
-from ..common.misc import lazy_value, OrderedDict
+from ..sprkkr.configuration import ConfigurationFile, ConfigurationSection
+from ..common.misc import OrderedDict
+from ..common.decorators import cached_class_property
 import shutil
 from typing import Union
 
+class InputSection(ConfigurationSection):
+    """ Input parameters sections has nothing special, yet. """
+
 def resolve_executable_postfix(postfix:Union[str,bool]):
     """" Return the postfix, that is appended after the name of SPR-KKR executable.
 
@@ -34,7 +39,7 @@ def resolve_executable_postfix(postfix:Union[str,bool]):
         return os.getenv('SPRKKR_EXECUTABLE_SUFFIX', '')
     return postfix
 
-class InputParameters(RootConfigurationContainer):
+class InputParameters(ConfigurationFile):
   """ It holds the configuration values for a SPR-KKR task and run the task
 
   This class is a ConfigurationContainer, thus, it holds the configuration values
@@ -57,10 +62,17 @@ class InputParameters(RootConfigurationContainer):
   _default_mpi_runner = None
 
   @classmethod
-  def default_mpi_runner(cls):
+  def default_mpi_runner(cls, auto:bool=False):
       """ Return the executable and its params to run a mpi task.
           The runner is determined by autodetection.
 
+      Parameters
+      ----------
+      auto:bool
+          If true,
+             - do not warn if no mpi is found
+             - check the number of available cpus, do not return a runner, if there is only one
+
       Return
       ------
       mpi_runner: list or False
@@ -73,8 +85,11 @@ class InputParameters(RootConfigurationContainer):
              if shutil.which(r):
                 cls._default_mpi_runner = [ r ]
                 return cls._default_mpi_runner
-         print("No MPI runner found. Disabling MPI!!!")
+         if not auto:
+             print("No MPI runner found. Disabling MPI!!!")
          cls._default_mpi_runner=False
+      if auto and hasattr(os, 'sched_getaffinity') and len(os.sched_getaffinity(0))==1:
+          return None
       return cls._default_mpi_runner
 
   def mpi_runner(self, mpi):
@@ -82,11 +97,14 @@ class InputParameters(RootConfigurationContainer):
 
       Parameters
       ----------
-      mpi_runner: Union[bool|str|list|int]
+      mpi_runner: Union[bool,str,list,int]
 
 
         - If True is given, return the default mpi-runner
         - If False is given, no mpi-runner is returned.
+        - If 'auto' is given, it is the same as True, however
+             * no warning is given if no mpi is found
+             * MPI is not used, if only one CPU is available
         - If a string is given, it is interpreted as a list of one item
         - If a list (of strings) is given, the user specified its own runner, use it as is
           as the parameters for subprocess.run.
@@ -108,6 +126,8 @@ class InputParameters(RootConfigurationContainer):
       if isinstance(mpi, list):
           return mpi
       if isinstance(mpi, str):
+          if mpi == 'auto':
+              return self.default_mpi_runner(auto=True) or None
           return [ mpi ]
       runner = self.default_mpi_runner()
       if not runner:
@@ -252,8 +272,7 @@ class InputParameters(RootConfigurationContainer):
       else:
           raise KeyError("No option with name {} in any of the members".format(name))
 
-  @staticmethod
-  @lazy_value
+  @cached_class_property
   def definitions():
       names = (i for i in pkgutil.iter_modules(definitions.__path__) if i.name != 'sections')
       im = importlib.import_module
@@ -263,7 +282,7 @@ class InputParameters(RootConfigurationContainer):
   @classmethod
   def is_it_a_input_parameters_name(cls, name):
       name = name.upper()
-      return name if name in cls.definitions() else False
+      return name if name in cls.definitions else False
 
   @classmethod
   def create_input_parameters(cls, arg):
@@ -304,7 +323,11 @@ class InputParameters(RootConfigurationContainer):
       input_parameters: InputParameters
         Input parameters with the default values for the given task.
       """
-      return InputParameters(cls.definitions()[name.upper()])
+      return InputParameters(cls.task_definition(name))
+
+  @classmethod
+  def task_definition(cls, task_name):
+      return cls.definitions[task_name.upper()]
 
   @classmethod
   def default_parameters(cls):
@@ -312,18 +335,22 @@ class InputParameters(RootConfigurationContainer):
       return cls.create('SCF')
 
   @classmethod
-  def from_file(cls, filename):
+  def from_file(cls, filename, allow_dangerous:bool=False):
       """ Read an input file and create a new InputParameters object from the readed stuff
 
       Parameters
       ----------
       filename: str or file
         Input file (either its filename, or an open file)
+
+      allow_dangerous
+        Allow to read dangerous values of options: i.e. the values that do not fullfil the
+        required type of the given option or its other requirements.
       """
-      definitions = cls.definitions()
+      definitions = cls.definitions
       for d in definitions.values():
           try:
-             out = d.read_from_file(filename)
+             out = d.read_from_file(filename, allow_dangerous=allow_dangerous)
              return out
           except Exception as e:
              last = e
@@ -340,5 +367,14 @@ class InputParameters(RootConfigurationContainer):
       out = out + ' for task ' + d.name.upper()
       return out
 
+  def change_task(self, task):
+      """ Change the task to the given task. Retain the value of the options,
+      that are present in the new task.
+      """
+      vals = self.to_dict()
+      self._definition = self.task_definition(task)
+      self._init_members_from_the_definition()
+      self.set(vals, unknown = 'ignore')
+
 #at least, to avoid a circular import
 from ..sprkkr import calculator
